/*
 *  Copyright (C) 2006 Mikhail A. Kryshen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: Define.java,v 1.7 2006/12/14 14:35:00 mikhail Exp $
 */

package kryshen.tema.functions;

import java.io.*;
import java.util.*;

import kryshen.tema.*;

/**
 * Define function implementation.
 *
 * @author Mikhail A. Kryshen
 */
public class Define extends Function {
    public static final Function DEFINE = new Define();

    public Define() {
    }

    public int invoke(FunctionDataParser fdp, Writer out)
        throws IOException, TemplateException {
        
        final String arg0 = fdp.getNextArg();
        final String data = fdp.getData();

        Function newFunction = new Function() {
                public int invoke(final FunctionDataParser fdp, final Writer out)
                    throws IOException, TemplateException {
		    
                    TemplateParser functionParser = 
                        new TemplateParser(fdp.getTemplateParser());
                    
                    functionParser.registerFunction
			("nextarg", new Function() {
				public int invoke(FunctionDataParser unusedFdp, 
						  final Writer out)
				    throws IOException, TemplateException {
				    
				    return fdp.parseNextArg(out);
				}
			    });
		    
                    functionParser.registerFunction
			("data",new Function() {
				public int invoke(FunctionDataParser unusedFdp, 
						  final Writer out)
				    throws IOException, TemplateException {
				    
				    return fdp.parseData(out);
				}
			    });
		    
                    TemplateReader dataReader = new TemplateReader
                        (new StringReader(data), fdp.getTemplateReader());

                    return functionParser.parse(dataReader, out);
                }
            };
	
        fdp.getTemplateParser().registerFunction(arg0, newFunction);
	
        out.write(arg0);
        return 1;
    }
}
