/*
 *  Copyright (C) 2006 Mikhail A. Kryshen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: Logics.java,v 1.6 2006/12/14 14:39:26 mikhail Exp $
 */

package kryshen.tema.functions;

import java.io.*;
import java.util.*;

import kryshen.tema.*;

/**
 * Logical and conditional functions.
 *
 * @author Mikhail A. Kryshen
 */
public class Logics {
    public static final Function FALSE =
        new Function() {
            public int invoke(FunctionDataParser fdp, Writer out)
                throws IOException, TemplateException {
		
		fdp.parseData(out);
                return 0;
            }
        };

    public static final Function TRUE =
        new Function() {
            public int invoke(FunctionDataParser fdp, Writer out)
                throws IOException, TemplateException {
		
		fdp.parseData(out);
                return 1;
            }
        };

    /**
     * Outputs it's data it has non-zero value.
     */
    public static final Function OPTIONAL =
        new Function() {
            public int invoke(FunctionDataParser fdp, Writer out)
                throws IOException, TemplateException {

                StringWriter sw = new StringWriter();
                int s = fdp.parseData(sw);
                sw.close();
                String data = sw.toString();

                if (s != 0) {
                    out.write(data);
                    return 1;
                } else {
                    return 0;
                }
            }
        };
}
